@@ -1,14 +1,16 @@
 /*
- * -----------------------------------------------------------------
- * $Revision: 1.9 $
- * $Date: 2010/12/01 22:30:43 $
  * ----------------------------------------------------------------- 
  * Programmer(s): Radu Serban @ LLNL
  * -----------------------------------------------------------------
- * Copyright (c) 2005, The Regents of the University of California.
- * Produced at the Lawrence Livermore National Laboratory.
+ * SUNDIALS Copyright Start
+ * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * and Southern Methodist University.
  * All rights reserved.
- * For details, see the LICENSE file.
+ *
+ * See the top-level LICENSE and NOTICE files for details.
+ *
+ * SPDX-License-Identifier: BSD-3-Clause
+ * SUNDIALS Copyright End
  * -----------------------------------------------------------------
  * This is the implementation file for the CVDIAG linear solver.
  * -----------------------------------------------------------------
@@ -36,7 +38,7 @@ static int CVDiagSetup(CVodeMem cv_mem, int convfail, N_Vector ypred,
 static int CVDiagSolve(CVodeMem cv_mem, N_Vector b, N_Vector weight,
                        N_Vector ycur, N_Vector fcur);
 
-static void CVDiagFree(CVodeMem cv_mem);
+static int CVDiagFree(CVodeMem cv_mem);
 
 
 /* 
@@ -89,7 +91,7 @@ static void CVDiagFree(CVodeMem cv_mem);
  * respectively.  It allocates memory for a structure of type
  * CVDiagMemRec and sets the cv_lmem field in (*cvode_mem) to the
  * address of this structure.  It sets setupNonNull in (*cvode_mem) to
- * TRUE.  Finally, it allocates memory for M, bit, and bitcomp.
+ * SUNTRUE.  Finally, it allocates memory for M, bit, and bitcomp.
  * The CVDiag return value is SUCCESS = 0, LMEM_FAIL = -1, or 
  * LIN_ILL_INPUT=-2.
  * -----------------------------------------------------------------
@@ -132,9 +134,6 @@ int CVDiag(void *cvode_mem)
 
   last_flag = CVDIAG_SUCCESS;
 
-  /* Set flag setupNonNull = TRUE */
-  setupNonNull = TRUE;
-
   /* Allocate memory for M, bit, and bitcomp */
     
   M = N_VClone(vec_tmpl);
@@ -378,8 +377,8 @@ static int CVDiagSetup(CVodeMem cv_mem, int convfail, N_Vector ypred,
     return(1);
   }
 
-  /* Set jcur = TRUE, save gamma in gammasv, and return */
-  *jcurPtr = TRUE;
+  /* Set jcur = SUNTRUE, save gamma in gammasv, and return */
+  *jcurPtr = SUNTRUE;
   gammasv = gamma;
   last_flag = CVDIAG_SUCCESS;
   return(0);
@@ -434,7 +433,7 @@ static int CVDiagSolve(CVodeMem cv_mem, N_Vector b, N_Vector weight,
  * -----------------------------------------------------------------
  */
 
-static void CVDiagFree(CVodeMem cv_mem)
+static int CVDiagFree(CVodeMem cv_mem)
 {
   CVDiagMem cvdiag_mem;
   
@@ -445,6 +444,8 @@ static void CVDiagFree(CVodeMem cv_mem)
   N_VDestroy(bitcomp);
   free(cvdiag_mem);
   cv_mem->cv_lmem = NULL;
+  
+  return(0);
 }
 
 
@@ -480,7 +481,7 @@ int CVDiagB(void *cvode_mem, int which)
   cv_mem = (CVodeMem) cvode_mem;
 
   /* Was ASA initialized? */
-  if (cv_mem->cv_adjMallocDone == FALSE) {
+  if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CVDIAG_NO_ADJ, "CVSDIAG", "CVDiagB", MSGDG_NO_ADJ);
     return(CVDIAG_NO_ADJ);
   } 
