@@ -3,15 +3,15 @@
  *                Aaron Collier @ LLNL
  *                David J. Gardner @ LLNL
  * -----------------------------------------------------------------
- * LLNS Copyright Start
- * Copyright (c) 2014, Lawrence Livermore National Security
- * This work was performed under the auspices of the U.S. Department 
- * of Energy by Lawrence Livermore National Laboratory in part under 
- * Contract W-7405-Eng-48 and in part under Contract DE-AC52-07NA27344.
- * Produced at the Lawrence Livermore National Laboratory.
+ * SUNDIALS Copyright Start
+ * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * and Southern Methodist University.
  * All rights reserved.
- * For details, see the LICENSE file.
- * LLNS Copyright End
+ *
+ * See the top-level LICENSE and NOTICE files for details.
+ *
+ * SPDX-License-Identifier: BSD-3-Clause
+ * SUNDIALS Copyright End
  * -----------------------------------------------------------------
  * This is the implementation file for the Fortran interface to
  * the KINSOL package. See fkinsol.h for usage.
@@ -28,8 +28,7 @@
 #include "fkinsol.h"     /* prototypes of interfaces and global vars. */
 #include "kinsol_impl.h" /* definition of KINMem type                 */
 
-#include <kinsol/kinsol_direct.h> /* prototypes for KINDLS interface routines   */
-#include <kinsol/kinsol_spils.h>  /* prototypes for KINSPILS interface routines */
+#include <kinsol/kinsol_ls.h> /* KINLS interface routine prototypes   */
 
 /*------------------------------------------------------------------
   definitions of global variables shared amongst various routines
@@ -38,7 +37,6 @@
 void *KIN_kinmem;
 long int *KIN_iout;
 realtype *KIN_rout;
-int KIN_ls;
 
 /*------------------------------------------------------------------
   private constants
@@ -241,37 +239,34 @@ void FKIN_SETVIN(char key_name[], realtype *vval, int *ier)
 }
 
 /*------------------------------------------------------------------
-  Function : FKIN_DLSINIT
+  Function : FKIN_LSINIT
   ------------------------------------------------------------------*/
 
-/* Fortran interface to C routine KINDlsSetLinearSolver */
-void FKIN_DLSINIT(int *ier) {
-  if ( (KIN_kinmem == NULL) || (F2C_KINSOL_linsol == NULL) ||
-       (F2C_KINSOL_matrix == NULL) ) {
-    *ier = -1;
-    return;
-  }
-  *ier = KINDlsSetLinearSolver(KIN_kinmem, F2C_KINSOL_linsol,
-                               F2C_KINSOL_matrix);
-  KIN_ls = KIN_LS_DIRECT;
-  return;
-}
-
-/*------------------------------------------------------------------
-  Function : FKIN_DLSINIT
-  ------------------------------------------------------------------*/
-
-/* Fortran interface to C routine KINSpilsSetLinearSolver */
-void FKIN_SPILSINIT(int *ier) {
+/* Fortran interface to C routine KINSetLinearSolver */
+void FKIN_LSINIT(int *ier) {
   if ( (KIN_kinmem == NULL) || (F2C_KINSOL_linsol == NULL) ) {
     *ier = -1;
     return;
   }
-  *ier = KINSpilsSetLinearSolver(KIN_kinmem, F2C_KINSOL_linsol);
-  KIN_ls = KIN_LS_ITERATIVE;
+  *ier = KINSetLinearSolver(KIN_kinmem, F2C_KINSOL_linsol,
+                            F2C_KINSOL_matrix);
   return;
 }
 
+/*------------------------------------------------------------------
+  Function : FKIN_DLSINIT -- DEPRECATED
+  ------------------------------------------------------------------*/
+
+void FKIN_DLSINIT(int *ier)
+{ FKIN_LSINIT(ier); }
+
+/*------------------------------------------------------------------
+  Function : FKIN_SPILSINIT -- DEPRECATED
+  ------------------------------------------------------------------*/
+
+void FKIN_SPILSINIT(int *ier)
+{ FKIN_LSINIT(ier); }
+
 /*------------------------------------------------------------------
   Function : FKIN_SOL
   ------------------------------------------------------------------*/
@@ -305,6 +300,13 @@ void FKIN_SOL(realtype *uu, int *globalstrategy,
   }
   N_VSetArrayPointer(fscale, fscalevec);
 
+  /* If using the fixed-point solver, initialize F2C_KINSOL_linsol 
+     and F2C_KINSOL_matrix to NULL */
+  if (*globalstrategy == KIN_FP) {
+    FKINNullMatrix();
+    FKINNullLinsol();
+  }
+  
   /* Call main solver function */
   *ier = KINSol(KIN_kinmem, uuvec, *globalstrategy, uscalevec, fscalevec);
 
@@ -326,26 +328,15 @@ void FKIN_SOL(realtype *uu, int *globalstrategy,
   KINGetFuncNorm(KIN_kinmem, &KIN_rout[0]);                  /* FNORM */
   KINGetStepLength(KIN_kinmem, &KIN_rout[1]);                /* SSTEP */
 
-  switch(KIN_ls) {
-
-  case KIN_LS_DIRECT:
-    KINDlsGetWorkSpace(KIN_kinmem, &KIN_iout[6], &KIN_iout[7]); /* LRW & LIW */
-    KINDlsGetLastFlag(KIN_kinmem, &KIN_iout[8]);                /* LSTF */
-    KINDlsGetNumFuncEvals(KIN_kinmem, &KIN_iout[9]);            /* NFE */
-    KINDlsGetNumJacEvals(KIN_kinmem, &KIN_iout[10]);            /* NJE */    
-    break;
-  case KIN_LS_ITERATIVE:
-    KINSpilsGetWorkSpace(KIN_kinmem, &KIN_iout[6], &KIN_iout[7]); /* LRW & LIW */
-    KINSpilsGetLastFlag(KIN_kinmem, &KIN_iout[8]);                /* LSTF */
-    KINSpilsGetNumFuncEvals(KIN_kinmem, &KIN_iout[9]);            /* NFE */
-    KINSpilsGetNumJtimesEvals(KIN_kinmem, &KIN_iout[10]);         /* NJE */
-    KINSpilsGetNumPrecEvals(KIN_kinmem, &KIN_iout[11]);           /* NPE */
-    KINSpilsGetNumPrecSolves(KIN_kinmem, &KIN_iout[12]);          /* NPS */
-    KINSpilsGetNumLinIters(KIN_kinmem, &KIN_iout[13]);            /* NLI */
-    KINSpilsGetNumConvFails(KIN_kinmem, &KIN_iout[14]);           /* NCFL */
-    break;
-
-  }
+  KINGetLinWorkSpace(KIN_kinmem, &KIN_iout[6], &KIN_iout[7]); /* LRW & LIW */
+  KINGetLastLinFlag(KIN_kinmem, &KIN_iout[8]);                /* LSTF */
+  KINGetNumLinFuncEvals(KIN_kinmem, &KIN_iout[9]);            /* NFE */
+  KINGetNumJacEvals(KIN_kinmem, &KIN_iout[10]);               /* NJE */    
+  KINGetNumJtimesEvals(KIN_kinmem, &KIN_iout[11]);            /* NJT */
+  KINGetNumPrecEvals(KIN_kinmem, &KIN_iout[12]);              /* NPE */
+  KINGetNumPrecSolves(KIN_kinmem, &KIN_iout[13]);             /* NPS */
+  KINGetNumLinIters(KIN_kinmem, &KIN_iout[14]);               /* NLI */
+  KINGetNumLinConvFails(KIN_kinmem, &KIN_iout[15]);           /* NCFL */
 
   return;
 }
@@ -360,7 +351,7 @@ void FKIN_FREE(void)
 
   kin_mem = (KINMem) KIN_kinmem;
 
-  /* free DLS/SPILS interface */
+  /* free LS interface */
   if (kin_mem->kin_lfree)
     kin_mem->kin_lfree(kin_mem);
   kin_mem->kin_lmem = NULL;
